<?php

if (!defined('_PS_VERSION_'))
	exit;

class GateWay
{
	private $apiUrl = 'http://behbank.com/ws';

	private $gatewayUrl = 'http://behbank.com/payment';

	public function paymentRequest($params){
		$curl = curl_init();
		curl_setopt($curl, CURLOPT_URL, $this->apiUrl.'/payments/request.json');
		curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type' => 'application/json'));
		curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($params));
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		$response = json_decode(curl_exec($curl));
		curl_close($curl);
		if(isset($response->status) && $response->status == 1 ){
			return array(
				'status'    		=>  true,
				'paymentNumber'     =>  $response->paymentNumber,
			);
		}else{
			return array(
				'status'    =>  false,
				'message'   =>  $response->message,
			);
		}
	}

	public function paymentVerify($params){
		$curl = curl_init();
		curl_setopt($curl, CURLOPT_URL, $this->apiUrl.'/payments/verify.json');
		curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type' => 'application/json'));
		curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode( $params ));
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		$response = json_decode(curl_exec($curl));
		curl_close($curl);
		if(isset($response->status) && $response->status == 1){
			return array(
				'status'    =>  true,
			);
		}else{
			return array(
				'status'    =>  false,
				'message'   =>  $response->message,
			);
		}
	}

	public function getGatewayUrl(){
		return $this->gatewayUrl;
	}
}

function epay123_config()
{
	return array(
		'FriendlyName' => array('Type' => 'System', 'Value'=>'BehBank'),
		'merchant' => array('FriendlyName' => 'merchant', 'Type' => 'text', 'Size' => '50'),
		'Currencies' => array('FriendlyName' => 'Currencies', 'Type' => 'dropdown', 'Options' => 'Rial,Toman')
	);	
}

class behbankpayment extends PaymentModule
{

	private $_html = '';
	private $_postErrors = array();


	public function __construct()
	{
		$this->name = 'behbankpayment';
		$this->tab = 'payments_gateways';
		$this->version = '1.9';
		$this->author = 'http://behbank.com';
		$this->currencies = true;
  		$this->currencies_mode = 'checkbox';
		parent::__construct();
		$this->page = basename(__FILE__, '.php');
		$this->displayName = 'درگاه پرداخت بهبانک';
		$this->description = 'پرداخت از طریق درگاه بهبانک';
		$this->confirmUninstall = 'آیا مطمئن هستید می خواهید جزئیات را حذف کنید ؟';
		if (!sizeof(Currency::checkPaymentCurrencies($this->id)))
			$this->warning = 'لطفا واحد پولی درگاه را تنظیم نمایید';
		$config = Configuration::getMultiple(array('gateway_merchant', ''));
		if (!isset($config['gateway_merchant']))
			$this->warning = 'لطفا اطلاعات درگاه را تنظیم نمایید';
	}

	public function install()
	{
		if (!parent::install() or !Configuration::updateValue('gateway_merchant', '') or !$this->registerHook('payment') or !$this->registerHook('paymentReturn')){
			return false;
		}
		return true;
	}

	public function uninstall()
	{
		if (!Configuration::deleteByName('gateway_merchant') or !parent::uninstall()){
			return false;
		}

		return true;
	}

	public function displayFormSettings()
	{
		$this->_html .= '<form action="'.$_SERVER['REQUEST_URI'].'" method="post">
			<fieldset>
				<legend><img src="../img/admin/cog.gif" alt="" class="middle" />Settings</legend>
				<label>BehBank Merchant Code</label>
				<div class="margin-form"><input type="text" size="50" name="gateway_merchant" id="gateway_merchant" value="'.Configuration::get('gateway_merchant').'" /></div>
				<center><input type="submit" name="submit" value="Update Settings" class="button" /></center>
			</fieldset>
		</form>';
	}

	public function displayConf()
	{
		$this->_html .= '<div class="conf confirm"><img src="../img/admin/ok.gif" />Settings updated</div>';
	}

	public function displayErrors()
	{
		foreach ($this->_postErrors as $err)
		$this->_html .= '<div class="alert error">'. $err .'</div>';
	}

	public function getContent()
	{
		$this->displayGateWayPayment();
		$this->postprocess();
		$this->displayFormSettings();
		return $this->_html;
	}
	
	private function postprocess()
	{
		if (isset($_POST['submit']))
		{
			if (empty($_POST['gateway_merchant']))
				$this->_postErrors[] = 'Merchant is required.';
			if (!sizeof($this->_postErrors))
			{
				Configuration::updateValue('gateway_merchant', $_POST['gateway_merchant']);
				return $this->displayConf();
			}
			return $this->displayErrors();
		}
	}

	private function displayGateWayPayment()
	{
		$this->_html .= '<h2>Behbank.com Payment</h2><img src="../modules/behbankpayment/behbank.png" style="float:left; margin-right:15px;"><b>This module allows you to accept payments by Online Payment Gateway.</b><br /><br />Any cart from Shetab Banks are accepted.<br /><br /><br />';
	}

	public function execPayment($cart)
	{
		global $cookie, $smarty;
		$rial_id = Currency::getIdByIsoCode('IRR');
		$purchase_currency = new Currency($rial_id);
		$current_currency = new Currency($cookie->id_currency);
		if($cookie->id_currency == $purchase_currency->id)
			$amount = number_format($cart->getOrderTotal(true, 3), 0, '', '');
		else
			$amount= number_format($this->convertPriceFull($cart->getOrderTotal(true, 3), $current_currency, $purchase_currency), 0, '', '');
		$redirect = (Configuration::get('PS_SSL_ENABLED') ?'https://' :'http://').$_SERVER['HTTP_HOST'].__PS_BASE_URI__.'modules/behbankpayment/validation.php';
		$redirect .= '?price=' . $amount . '&code=' . $cart->id;
		$gateway = new GateWay();
		$params = array(
			'merchantCode'  => Configuration::get('gateway_merchant'),
			'amount'        => $amount,
			'callBackUrl'   => $redirect,
			'invoiceNumber' => $cart->id
		);
		$request = $gateway->paymentRequest($params);
		if (!$request['status']) {
			return '<p><font color="red">خطا در اتصال به درگاه پرداخت - ' . $request['message'] . '</font></p>';
		}
		setcookie("OrderId", $cart->id, time()+1800);
		setcookie("PurchaseAmount", $amount, time()+1800);
		return 'در حال انتقال به درگاه پرداخت، لطفا چند لحظه صبر کنید<script>window.location = "'.$gateway->getGatewayUrl().'/'.$request['paymentNumber'].'";</script>';
		/*return '<p>در حال اتصال به بانک</p>
				<form name="frmPay" method="get" action="'.$gateway->paymentGateway($request['paymentNumber']).'">
					<center>
					<noscript>
					<p style="font:bold 12px tahoma; color:#FF0000; direction:rtl; text-align:center;">در صورتیکه بعد از گذشت 5 ثانیه به درگاه بانکی وصل نشده اید ؛ بر روی دکمه ی زیر کلیک کنید.</p>
					<input type="submit" value="پرداخت" style="font:bold 12px tahoma; color:#FF0000; direction:rtl; text-align:center;" />
					</noscript>
					</center>
				</form>';*/
	}

	public function confirmPayment($result2)
	{
		if (isset($_POST['status']) && $_POST['status'] == 1)
		{
			$gateway = new GateWay();
			$params = array(
				'merchantCode'  => Configuration::get('gateway_merchant'),
				'paymentNumber' => $_POST['paymentNumber']
			);
			$request = $gateway->paymentVerify($params);
			if ($request['status'])
			{
				return true;
			}
		}
		return false;
	}

	public function showMessages($result)
	{
		$this->_postErrors[] = $result;
		$this->displayErrors();
		echo $this->_html;
		return $result;
	}

	public function hookPayment($params)
	{
		if (!$this->active)
		return;
		return $this->display(__FILE__, 'payment.tpl');
	}

	public function hookPaymentReturn($params)
	{
		if (!$this->active)
		return;
		return $this->display(__FILE__, 'confirmation.tpl');
	}

	/**
	 *
	 * Convert amount from a currency to an other currency automatically
	 * @param float $amount
	 * @param Currency $currency_from if null we used the default currency
	 * @param Currency $currency_to if null we used the default currency
	 */
	public static function convertPriceFull($amount, Currency $currency_from = null, Currency $currency_to = null)
	{
		if ($currency_from === $currency_to)
			return $amount;

		if ($currency_from === null)
			$currency_from = new Currency(Configuration::get('PS_CURRENCY_DEFAULT'));

		if ($currency_to === null)
			$currency_to = new Currency(Configuration::get('PS_CURRENCY_DEFAULT'));

		if ($currency_from->id == Configuration::get('PS_CURRENCY_DEFAULT'))
			$amount *= $currency_to->conversion_rate;
		else
		{
			$conversion_rate = ($currency_from->conversion_rate == 0 ? 1 : $currency_from->conversion_rate);
			// Convert amount to default currency (using the old currency rate)
			$amount = Tools::ps_round($amount / $conversion_rate, 2);
			// Convert to new currency
			$amount *= $currency_to->conversion_rate;
		}
		return Tools::ps_round($amount, 2);
	}

}
